#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import system modules
import os, fnmatch
import math

# Import pygame modules
import pygame
from pygame.locals import *
import fontmgr
from evinput import Input
import sprite
import map
import mapeditor
import game

# Enviroment variables
os.environ['SDL_VIDEO_WINDOW_POS'] = "center"

# Game constants
SCREENRECT   = Rect(0, 0, 800, 600)
SCREEN_FLAGS = 0
MUSIC_VOL = 60
SOUND_VOL = 80
MAX_FPS = 60 

CONTROLS = {
    "up"    : K_UP,
    "right" : K_RIGHT,
    "down"  : K_DOWN,
    "left"  : K_LEFT,
    "action": K_SPACE,
    "run"   : K_z,
    "attack": K_x,
    "next_feather": K_s,
    "prev_feather": K_a
}

OP_TABLE = ["*","+","-","/","%"]
EQ_COLOR_TABLE = [ (255,0,0,255), (255,255,0,0), (0,255,0,255) ]
mathExp = None

# Global game variables
screen = None
clock = None
fonts = None
forceUpdate = False
quitPrompt = False

# The sound that is currently playing. So if the sound we are playing is the
# same as the new sound to play we don't need to stop the sound.
playing_s = ""

# The background musics for specific areas
game_musics = {}

# The images we use in more than one screen
game_images = { }

# The sound effects we play in game
game_sounds = { }

# Sprite groups
render_group = sprite.oSpriteGroup()
baddys_group = sprite.oSpriteGroup()
attack_group = sprite.oSpriteGroup()
level = None

def updateScreen(dirtyRects=None):
    global forceUpdate
    if not forceUpdate and dirtyRects is not None:
        pygame.display.update(dirtyRects)
    else:
        pygame.display.flip()
        
    forceUpdate = False
    
# This function play a music file from the disk
def play_music(file, directory=os.curdir, loop=-1, queue=False):
    global playing_s
    if not pygame.mixer:
        return
        
    # verify if the music exists in our dictionary
    if game_musics.has_key(file):
        file = game_musics[file]
        
    # create the path for the file
    elif file is not None:
        file = os.path.join(directory, file)
    
    # if we are current playing the file don't stop and play again
    if file == playing_s:
        return
        
    # I had some problems using queue and XM files, so I decided using just a simple fadeout effect
    if queue:
        pygame.mixer.music.fadeout(1000)
        
    # adjust the playing file
    playing_s = file or ""
    
    if file is None:
        if queue:
            pygame.time.wait(1000)
        return
    
    # play the file
    pygame.mixer.music.stop()          
    pygame.mixer.music.load(file)
    pygame.mixer.music.play(loop)


def play_sound(name):
    if not game_sounds.has_key(name):
        raise "The sound doesn't exists"
        return
    ## we need to set the main volume before we play
    game_sounds[name].set_volume(SOUND_VOL/100.0)
    game_sounds[name].play()

# This function loads a sound file
def load_sound(directory,file):
    if not pygame.mixer: return dummysound()
    file = os.path.join(directory, file)
    try:
        sound = pygame.mixer.Sound(file)
        return sound
    except pygame.error:
        raise 'Warning, unable to load,', file

# Loads an image file
def load_image(file, directory=None, auto_alpha=True):
    """Loads an image file."""
    # If a directory is given, generate the path for the file
    if directory is not None:
        file = os.path.join( directory, file )
        
    # Try to load the image
    try:
        surface = pygame.image.load( file )
    except:
        raise SystemExit, 'Can\'t load image "%s" %s' % ( file, pygame.get_error() )
    
    # Verify if image is PNG to use per-pixel alpha
    if os.path.splitext( file )[1].lower() == ".png" and auto_alpha:
        return surface.convert_alpha()
    else:
        return surface.convert()
    
# A simple dissolve transition...
def dissolve_transition(old_screen, new_screen, time, rect=None):
    # initialize values
    alpha = 0
    # calculate how much we will increase alpha
    incAlpha = 255/float(time)

   
    while alpha < 255:        
        lag = clock.tick(MAX_FPS)
                
        if handleEvents() == "quit":
            exit()
        
        if Input.up(K_ESCAPE):
            new_screen.set_alpha(None)
            screen.blit(new_screen, (0,0))
            updateScreen()
            return
                
        # blit the old and new screens
        screen.blit(old_screen, (0,0))
        new_screen.set_alpha(int(alpha))
        screen.blit(new_screen, (0,0))
        updateScreen()
                
        # increase alpha
        alpha += incAlpha * lag
                
    # blit the new screen with full opacity
    new_screen.set_alpha(None)
    screen.blit(new_screen, (0,0))
    updateScreen()
    
def initializeFonts():
    global fonts
    fonts = fontmgr.cFontManager(((None, 24), (None, 36), ('arial', 24)))
      
def handleEvents(events=None):
    global next_screen, forceUpdate, quitPrompt
    
    # update last events
    Input.update()
    
    # Update our mouse position and 3 button state
    Input.update_mouse(pygame.mouse.get_pressed(), pygame.mouse.get_pos())
        
    # Add all events from the event queue to the input module
    # for this loop cycle.
    if events is None:
        events = pygame.event.get()
        
    Input.add_events(events)
    
    if Input.down(K_ESCAPE) or Input.isset("QUIT"):
        if not quitPrompt:
            quitPrompt = True
            if game.yesno_prompt("Exit RED -spider lily-?"):
                next_screen = None
                return "quit"
            quitPrompt = False
            
            forceUpdate = True
            Input.update()
    
    if Input.up(K_F8):
        mapeditor.map_editor()
        forceUpdate = True
    
def getInput(label, rect, default="", font=None, size=24, color=(255,255,255,255),
label_color=(255,0,255,255), align_h='left', align_v='top', antialias=False):
    result = default
    cursor = "_"
    cursor_time = 0
    last_screen = screen.copy().convert()
    new_screen = last_screen.copy().convert()
    new_screen.fill((0,0,0,127), rect=rect)
    fonts.Draw(new_screen, font, size, label, rect.move(0, -size-5), label_color, antialias=antialias)
    fonts.Draw(new_screen, font, size, default+cursor, rect, color, align_h, align_v, antialias)
    update_rect = rect.union(rect.move(0,-size-5))
    dissolve_transition(last_screen, new_screen, 250, update_rect)
    new_screen.fill((0,0,0,127), rect=rect)
    
    while True:
        lag = clock.tick(MAX_FPS)
        cursor_time += lag
        if cursor_time > 250:
            cursor = "_" if cursor == "" else ""
            cursor_time = 0
        
        for event in pygame.event.get():
            if event.type == QUIT:
                exit()
            if event.type == KEYDOWN:
                if event.key == K_RETURN:
                    new_screen = screen.copy().convert()
                    dissolve_transition(new_screen, last_screen, 250, None)
                    return result
                if event.key == K_ESCAPE:
                    new_screen = screen.copy().convert()
                    dissolve_transition(new_screen, last_screen, 250, None)
                    return default
                if event.key == K_BACKSPACE or event.key == K_DELETE:
                    lpn = len(result)-1
                    if lpn > 0:
                        result = result[:lpn]
                    else:
                        result = ""
                elif not event.key in (K_RETURN, K_ESCAPE, K_PAUSE, K_CLEAR) and not event.key >= K_UP:
                    result += event.unicode
    
        screen.blit(new_screen, (0,0))
        fonts.Draw(screen, font, size, label, rect.move(0, -size-5), label_color, antialias=antialias)
        fonts.Draw(screen, font, size, result+cursor, rect, color, align_h, align_v, antialias) 
        updateScreen([update_rect])
    
# crop a image
def im_crop(surface, rect):
    # verify bounds...
    w, h = surface.get_size()
    if rect.left + rect.width >= w:
        rect.width = w - rect.left
        if rect.width <= 0:
            rect.width = 0
            rect.left = 0
    if rect.top + rect.height >= h:
        rect.height = h - rect.top
        if rect.height <= 0:
            rect.height = 0
            rect.top = 0
    return surface.subsurface(rect)
    
    
def handlePlayerControls(char, lag):
    d = ""
    ly = char.y
    lx = char.x
    if Input.motion(CONTROLS["up"]) and char.jump <= 0 and char.jumping <= 0:
        char.y -= char.speed*char.speedMul * lag
        d += "N"
    if Input.motion(CONTROLS["down"]) and char.jump <= 0 and char.jumping <= 0:
        char.y += char.speed*char.speedMul * lag
        d += "S"
    if Input.motion(CONTROLS["left"]):
        char.x -= char.speed*char.speedMul * lag
        d += "W"
    if Input.motion(CONTROLS["right"]):
        char.x += char.speed*char.speedMul * lag
        d += "E"
    
    if d == "":
        char.changeState("stopped")
    else:
        char.changeState("walking")
        char.changeDirection(d)
    
    if not char.canWalk():
        char.x = lx
        char.y = ly
    
    if Input.motion(CONTROLS["attack"]):
        char.attacks[char.use_attack].attack(char)
        
    if Input.motion(CONTROLS["prev_feather"]):
        mc.use_attack -= 1
        if mc.use_attack < 0:
            mc.use_attack = len(mc.attacks)-1
            
    if Input.motion(CONTROLS["next_feather"]):
        mc.use_attack += 1
        if mc.use_attack >= len(mc.attacks):
            mc.use_attack = 0
    
    if Input.motion(CONTROLS["run"]):
        char.speedMul = 2
    else:
        char.speedMul = 1
    
    if Input.up(CONTROLS["action"]):
        w, h = level.tileSize
        x = int( math.floor( char.x / w ) )
        y = int( math.floor( (char.y-33) / h ) )# fix tiles have a blank area on top o.o
        tile = level.getTile(x, y)
        print y * level.width + x
        if tile.op > -1:
            tile.active = not tile.active
            if tile.active:
                play_sound("block_activate")
            else:
                play_sound("block_deactivate")
                
            tile.changed = True
            mathExp.activateTerm(x, y)
            level.activateTiles(mathExp.getResult())
            level.genBitmask()
            
    
def fileMatch(filename, patterns):
    for p in patterns:
        if fnmatch.fnmatch(filename, p):
            return True
    return False

def filterFiles(files, pattern):
    for file in files:
        if fileMatch(file, pattern.split(";")):
            yield file

def locate(pattern, root=os.curdir):
    for path, dirs, files in os.walk(os.path.abspath(root)):
        for filename in filterFiles(files, pattern):
            yield os.path.join(path, filename)
            
            