#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import pygame modules
import pygame
from pygame.locals import *

import map
import core
import sprite
import fontmgr
from evinput import Input
import time
import featheratk

class oStatsDisp:
    def __init__(self, iValue, position, **kwargs):
        self.value = iValue
        self.x, self.y = position
        self.image = kwargs.pop("image", None)
        self.font  = kwargs.pop("font" , None)
        self.size  = kwargs.pop("size" , None)
        self.aalias= kwargs.pop("antialias", True)
        self.color = kwargs.pop("color", (255,255,255,255))
        self.useNumbers = kwargs.pop("useNumbers", False)
        self.changed = True
        self.formatValue = kwargs.pop("formatValue", str)
        self.last_rects = [ ]
        
    def update(self, iValue, forceUpdate=False):
        if iValue != self.value or forceUpdate:
            self.value = iValue
            self.changed = True
            
    def draw(self, surface):
        rects = [ ]
        # assume we have a image to draw for each value =D
        if self.image is not None:
            w = self.image.get_width()
            for i in range(self.value):
                rects.append( surface.blit(self.image, (self.x + w*i, self.y)) )
        
        # otherwise we just renderize a font for it =D
        else:
            if self.useNumbers:
                x = self.x
                if self.x < 0:
                    x = (self.x*-1) * (800-core.numbers.getWidth( self.formatValue(self.value) ))
                    
                rect = core.numbers.draw(surface, (x, self.y), self.formatValue(self.value))
            else:
                rect = core.fonts.Draw(surface, self.font, self.size, self.formatValue(self.value),
                (self.x, self.y), self.color, antialias=self.aalias) 
                
            rects.append(rect)
             
        lr = self.last_rects[:]
        self.last_rects = rects[:]
        rects.extend(lr)
        return rects
    
class oPortalEquation():
    def __init__(self, mapa, pos):
        self.equation = []
        self.paths    = []
        self.portalN  = mapa.portalN
        self.terms    = 0
        self.state    = 0
        self.changed  = False
        self.mapa     = mapa
        self.x, self.y= pos
        self.last_rect= None
        self.loadData()
        
    def loadData(self):
        for index, tile in enumerate(self.mapa.mapData):
            if tile is not None:
                # path
                if tile.op == -1 and tile.number > -1 and not tile.number in self.paths:
                    self.paths.append(tile.number)
                # term
                elif tile.op > -1 and tile.number > -1:
                    self.terms += 1
                    
    def activateTerm(self, x, y):
        index = y * self.mapa.width + x
        tile  = self.mapa.getTile(x, y)
        # check if it's already activated (ugly, I know)
        for term in self.equation:
            if term[0] == index:
                self.equation.remove(term)
                break
            
        if tile.active:
            self.equation.append( [index, tile.number, tile.op] )
            
        ee = self.getResult()
        if ee == self.portalN:
            self.state = 2
        elif ee in self.paths:
            self.state = 1
        else:
            self.state = 0
        self.changed = True

    def evalEquation(self):
        result  = ""
        last_op = ""
        for term in self.equation:
            index, number, op = term
            result += last_op + str(number)
            last_op = core.OP_TABLE[op]
        return result
    
    def getResult(self):
        r = self.evalEquation()
        if r == "":
            return -1
        else:
            return eval(r)
    
    def draw(self, surface):
        color = core.EQ_COLOR_TABLE[self.state]
        eq = str(self.portalN) + " = " + self.evalEquation()
        if len(self.equation) == 0:
            eq += "?"
        
        rect = core.numbers.draw( surface, (self.x, self.y), eq )
        lr = self.last_rect
        self.last_rect = rect
        return [lr, rect]

def load_level(levelN, mc):
    core.level.load("map/level_" + str(levelN) + ".rmap")
    core.mathExp = oPortalEquation(core.level, (20, 545))
    px, py = core.level.findPortal()
    tw, th = core.level.tileSize
    mc.x = px*tw + mc.hotspot()[0]
    mc.y = py*th + mc.hotspot()[1]
    
    if levelN == 0:
        core.level.nests = [
            map.oNest(3, 11, "S", core.level, core.baddys_group),
            map.oNest(5, 116, "E", core.level, core.baddys_group)
        ]

def restart_level(mc, lives, gameOver=True):
    if lives.value == 0 and gameOver:
        core.next_screen = game_over
        return True
    px, py = core.level.findPortal()
    tw, th = core.level.tileSize
    mc.x = px*tw + mc.hotspot()[0]
    mc.y = py*th + mc.hotspot()[1]

def format_level_time(value):
    return str( int(value) )

def format_score(value):
    return str(value)[:9].zfill(9)
    

def main_game():
    core.play_music(None, queue=True)
    
    core.baddys_group.clear()
    core.render_group.clear()
    core.attack_group.clear()
    
    # load bg
    sky_bg = core.load_image("sky_bg.png", directory="gfx", auto_alpha=False)
    exp_bgs = core.load_image("exp_bg.png", directory="gfx")
    exp_bg = sprite.oSprite()
    exp_bg.x = 10
    exp_bg.y = 545
    exp_bg.animations = {
        "stopped": {
            "N": [0, True, [
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,0,278,33))),
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,33,278,33))),
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,66,278,33)))
            ]]
        }
    }
    exp_bg.changeState("stopped")
    core.render_group.add(exp_bg)
    core.numbers = fontmgr.oSFont(core.load_image("numbers.png", directory="gfx"),
        ["0","1","2","3","4","5","6","7","8","9","-","+","/","*","%",":","=","?","!"])
       
    # initialize map
    core.level = map.oMap()

    # current stats
    level = 0
    level_time = oStatsDisp(core.level.time, (700,20), useNumbers=True, formatValue=format_level_time)
    lives = oStatsDisp(3, (10, 20), image=core.load_image("haato.png", directory="gfx"))
    score = oStatsDisp(0, (-.5,20), useNumbers=True, formatValue=format_score)

    # main character
    mc = sprite.oSprite()
    
    mc_sprite = core.load_image("nozomi.png", directory="gfx")
    mc_mask   = core.load_image("nozomi_mask.png", directory="gfx")
    
    mc.animFromStrip( mc_sprite, [
        ["stopped","S",1,1,0,True, (23,45), (23,33)],
        ["stopped","W",4,1,0,True, (23,45), (17,33)],
        ["stopped","E",7,1,0,True, (23,45), (30,33)],
        ["stopped","N",10,1,0,True,(23,45), (23,33)],
        ["walking","S",0,3,.02,True, (23,45), (23,33)],
        ["walking","W",3,3,.02,True, (23,45), (17,33)],
        ["walking","E",6,3,.02,True, (23,45), (30,33)],
        ["walking","N",9,3,.02,True, (23,45), (23,33)] 
    ], 48, 48, mc_mask )
    
    mc.changeState("stopped")
    mc.freezeAttack = 0
    mc.AI = core.handlePlayerControls
    core.render_group.add(mc)
    
    mc.attacks = [ 
        featheratk.oBasicAttack(core.load_image("debug_feather.png", directory="gfx"))
    ]
    mc.use_attack = 0
    
    # load map
    load_level(level, mc)
    core.MAX_LEVEL = len( list(core.locate('*.rmap', './map')) )
    
    
    forceUpdate = True
    
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        if lag <= 0:
            lag = 1
        
        level_time.update(level_time.value - lag/1000.0)
        if level_time.value < 0:
            lives -= 1
            if restart_level(mc, lives): return
        
        if core.handleEvents() == "quit":
            return
                        
        dirty_rects = core.render_group.update(lag)
        core.level.updateNests(lag)
        exp_bg.animFrame = core.mathExp.state
        
        # check collisions
        # bugs
        if core.render_group.checkCollision(False, False, mc, core.baddys_group):
            core.play_sound("mc_dying")
            lives.update(lives.value-1)   
            if restart_level(mc, lives): return
                
        # feather attacks
        kills = core.attack_group.checkCollision(True, True, None, core.baddys_group)
        score.update(score.value + 100*len(kills))
        for i in kills:
            core.play_sound("bug_dying")
        
        # draw level / sprites
        core.screen.blit(sky_bg, (0,0))
        dirty_rects.extend( core.level.draw(core.screen) )
        core.render_group.draw(core.screen)
        
        # draw stats
        # lives
        dirty_rects.extend( lives.draw(core.screen) )
        # math exp
        dirty_rects.extend( core.mathExp.draw(core.screen) )
        # time
        dirty_rects.extend( level_time.draw(core.screen) )
        # score
        dirty_rects.extend( score.draw(core.screen) )
        # attacks
        for i, attack in enumerate(mc.attacks):
            attack.update(lag)
            dirty_rects.extend( attack.draw(core.screen, i == mc.use_attack) )
                
        dirty_rects.append( core.fonts.Draw(core.screen, 'arial', 24,
         str(core.clock.get_fps()), (650, 50), (255,255,255,255)))
        
        if forceUpdate:
            dirty_rects = None
        forceUpdate = False
        core.updateScreen(dirty_rects)
        
        
def game_over():
    core.play_music("game over", queue=True)
    
    # load game over screen
    goscreen = core.load_image("game_over.png", directory="gfx", auto_alpha=False)
    core.dissolve_transition( core.screen.copy().convert(), goscreen.copy().convert(), 1000 )
    
    while True:
        core.clock.tick(core.MAX_FPS)
        
        events = pygame.event.get()
        
        if core.handleEvents(events) == "quit":
            exit()
            
        for event in events:
            if event.type == KEYUP:
                core.next_screen = main_menu
                return
        
        core.screen.blit(goscreen, (0,0))                        
        core.updateScreen()
    
def credits_screen():
    goscreen = core.load_image("credits.png", directory="gfx", auto_alpha=False)
    core.dissolve_transition( core.screen.copy().convert(), goscreen.copy().convert(), 1000 )
    
    while True:
        core.clock.tick(core.MAX_FPS)
        
        events = pygame.event.get()
        
        if core.handleEvents(events) == "quit":
            exit()
            
        for event in events:
            if event.type == KEYUP:
                core.play_sound("button_click")
                core.next_screen = main_menu
                return
        
        core.screen.blit(goscreen, (0,0))                        
        core.updateScreen()
    
def yesno_prompt(text):
    last_screen = core.screen.copy().convert()
    ls = last_screen.copy().convert()
    bg = core.load_image("yesno_prompt.png", directory="gfx")
    selected = 0
    button_group = sprite.oSpriteGroup()
    def genButton(images, pos):
        button = sprite.oSprite()
        button.x = pos[0]
        button.y = pos[1]
        button.animations = {
            "active": {
                "N": [0,True,[sprite.oFrame(images[0])]]
            },
            "inactive": {
                "N": [0,True,[sprite.oFrame(images[1])]]
            }
        }
        button_group.add(button)
        return button
    
    selected = 0
    buttons = []
    buttons.append( genButton(core.game_images["no"], (200,300)) )
    buttons.append( genButton(core.game_images["yes"], (400,300)) )
    forceUpdate = True
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        
        if core.handleEvents() == "quit":
            exit()
                    
        if Input.up(K_LEFT):
            core.play_sound("cursor_move")
            selected -= 1
            if selected < 0:
                selected = len(buttons)-1
        if Input.up(K_RIGHT):
            core.play_sound("cursor_move")
            selected += 1
            if selected >= len(buttons):
                selected = 0
        if Input.up(K_RETURN):
            core.play_sound("button_click")
            core.dissolve_transition(core.screen.copy().convert(), ls, 1000)
            return bool(selected)
        
        for i, button in enumerate(buttons):
            button.changeState("active" if i == selected else "inactive")
                    
        dirty_rects = button_group.update(lag)
        core.screen.blit(ls, (0,0))
        core.screen.blit(bg, (0,0))
        button_group.draw(core.screen)        
        
        dirty_rects.append(core.fonts.Draw(core.screen, None, 36, text,
         pygame.Rect(50,250,700,50), (255,255,255,255), alignHoriz='center', antialias=True))        
        
        if last_screen is not None:
            core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
            last_screen = None
        
        core.updateScreen(dirty_rects)
 
def exit_screen():
    if yesno_prompt("Exit RED -spider lily-?"):
        core.next_screen = None
        return
    else:
        core.next_screen = main_menu
        return
    
def main_menu():
    core.play_music("main menu")
    # load bg
    bg = core.load_image("main_menu.png", directory="gfx", auto_alpha=False)
    selected = 0
    
    def genButton(images, pos):
        button = sprite.oSprite()
        button.x = pos[0]
        button.y = pos[1]
        button.animations = {
            "active": {
                "N": [0,True,[sprite.oFrame(images[0])]]
            },
            "inactive": {
                "N": [0,True,[sprite.oFrame(images[1])]]
            }
        }
        core.render_group.add(button)
        return button
    
    selected = 0
    buttons = []
    core.render_group.clear()
    for i, button in enumerate(["new game","options","help","credits","exit"]):
        y = 150 + i*40
        x = 75
        buttons.append( genButton(core.game_images[button], (x,y)) )
    
    menu_screens = [
        main_game,      # new game
        main_menu,      # options
        main_menu,      # help
        credits_screen, # credits
        exit_screen,    # exit
    ]
    
    forceUpdate = True
    last_screen = core.screen.copy().convert()
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        
        if core.handleEvents() == "quit":
            exit()
                    
        if Input.up(K_UP):
            core.play_sound("cursor_move")
            selected -= 1
            if selected < 0:
                selected = len(buttons)-1
        if Input.up(K_DOWN):
            core.play_sound("cursor_move")
            selected += 1
            if selected >= len(buttons):
                selected = 0
        if Input.up(K_RETURN):
            core.next_screen = menu_screens[selected]
            core.play_sound("button_click")
            return
        
        for i, button in enumerate(buttons):
            button.changeState("active" if i == selected else "inactive")
                    
        dirty_rects = core.render_group.update(lag)
        core.screen.blit(bg, (0,0))
        core.render_group.draw(core.screen)        
        
        if last_screen is not None:
            core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
            last_screen = None
        
        if forceUpdate:
            dirty_rects = None
        core.updateScreen(dirty_rects)
    
    
    
    