#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import system modules
import random
import os.path

# Initialize random number generation
random.seed()

# Import pygame modules
import pygame
from pygame.locals import *

# import higanbana specific modules
import core
import game

# Verify if we can use more than the standard BMP
if not pygame.image.get_extended():
    raise SystemExit, "Extended image module not found."
    
# The game starts here!
def main():
    # pre-initialize sound
    pygame.mixer.pre_init(0,0,0,1024)
    
    # Initializes pygame
    pygame.init()
    
    # Verify if we can play sounds
    if pygame.mixer and not pygame.mixer.get_init():
        print 'Warning, no sound'
        pygame.mixer = None
        
    # We need to set up the window too
    pygame.display.set_caption('RED -spider lily-')
        
    # Set the display mode
    bestdepth = pygame.display.mode_ok(core.SCREENRECT.size, core.SCREEN_FLAGS, 32)
    core.screen = pygame.display.set_mode(core.SCREENRECT.size, core.SCREEN_FLAGS, bestdepth)
    
    core.screen.fill((0,0,0,255))
    
    pygame.mixer.music.set_volume(core.MUSIC_VOL/100.0)
    
    pygame.display.update()
    
    core.clock = pygame.time.Clock()
    core.initializeFonts()
    
    # load sounds
    core.game_sounds["block_activate"]  = core.load_sound("sfx", "step_activate.wav")
    core.game_sounds["block_deactivate"]= core.load_sound("sfx", "step_deactivate.wav")
    core.game_sounds["feather_throw"]   = core.load_sound("sfx", "feather_throw.wav")
    core.game_sounds["portal_activate"] = core.load_sound("sfx", "portal_activate.wav")
    core.game_sounds["button_click"]    = core.load_sound("sfx", "button_click.wav")
    core.game_sounds["mc_dying"]        = core.load_sound("sfx", "mc_dying.wav")
    core.game_sounds["bug_dying"]       = core.load_sound("sfx", "bug_dying.wav")
    core.game_sounds["menu_cancel"]     = core.load_sound("sfx", "menu_cancel.wav")
    core.game_sounds["cursor_move"]     = core.load_sound("sfx", "cursor_move.wav")
    
    # music mappings
    core.game_musics = {
        "main menu": "bgm/main_menu.mp3",
        "game over": "bgm/game_over.mp3"
    }
    
    # load game buttons
    def getButtonImage(surface, button):
        y = button * 40
        inactive = core.im_crop(surface, pygame.Rect(0, y, 200, 40))
        active = core.im_crop(surface, pygame.Rect(200, y, 200, 40))
        return active, inactive
    
    buttons = core.load_image("buttons.png", directory="gfx")
    core.game_images["new game"] = getButtonImage(buttons, 0)
    core.game_images["options"]  = getButtonImage(buttons, 1)
    core.game_images["help"]     = getButtonImage(buttons, 2)
    core.game_images["credits"]  = getButtonImage(buttons, 3)
    core.game_images["exit"]     = getButtonImage(buttons, 4)
    core.game_images["yes"]      = getButtonImage(buttons, 5)
    core.game_images["no"]       = getButtonImage(buttons, 6)
        
    core.next_screen = game.main_menu
        
    while True:
        if core.next_screen is not None:
            core.next_screen()
        else:
            return

# call the "main" function if running this script
if __name__ == '__main__': main()