#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import pygame modules
import pygame
from pygame.locals import *

import map
import math
import core
import sprite
from evinput import Input

def new_map():
    ir = pygame.Rect(0, 550, 800, 50)
    mw = int( core.getInput("Map width:", ir, default="10") )
    mh = int( core.getInput("Map height:", ir, default="10") )
    pn = int( core.getInput("Portal number:", ir, default="0") )
    filename = core.getInput("Map filename:", ir, default="new_map.rmap")
    data = [ None for i in range(mw*mh) ]
    return map.oMap(width=mw, height=mh, portal=pn, mapData=data), filename

def load_map():
    ir = pygame.Rect(0, 550, 800, 50)
    filename = core.getInput("Map filename:", ir, default="new_map.rmap")
    mapa = map.oMap()
    mapa.load(filename)
    return mapa, filename

def edit_map(mapa):
    mw = int( core.getInput("Map width:", ir, default=str(mapa.width)) )
    mh = int( core.getInput("Map height:", ir, default=str(mapa.height)) )
    pn = int( core.getInput("Portal number:", ir, default=str(mapa.portalN)) )
    mapa.portalN = pn
    data = []
    for y in range(mapa.height):
        if y >= mh:
            break
        for x in range(mapa.width):
            if x >= mw:
                break
            data.append(mapa.getTile(x,y))
        for x in range(mw - mapa.width):
            data.append(None)
    for y in range(mh-mapa.height):
        data.append(None)
    
    mapa.width = mw
    mapa.height = mh
    return mapa

def map_editor():
    
    mapa, filename = new_map()
    
    tiles = [
        map.oTile(0, 25, True),
        map.oTile(1, 25, True),
        map.oTile(2, 25, True),
        map.oTile(3, 25, True),
        map.oTile(4, 25, False),
        map.oTile(5, 25, False),
        map.oTile(6, 25, True)
    ]
        
    selector = core.load_image("selector.png", directory="gfx")
    current_tile = 0
    number = 0
    op = 0
    task = 0
    ox = 0
    oy = 0
    tw, th = mapa.tileSize

    while True:
        lag = core.clock.tick(core.MAX_FPS)
        
        
        mx, my = pygame.mouse.get_pos()
        ox = int(math.floor( mx / tw ))
        oy = int(math.floor( my / th ))
        if ox >= mapa.width:
            ox = mapa.width-1
        if oy >= mapa.height:
            oy = mapa.height-1
        if ox < 0:
            ox = 0
        if oy < 0:
            oy = 0
        
        events = pygame.event.get()
        core.handleEvents(events)
        for event in events:
            if event.type == QUIT:
                core.MAP_EDITOR = False
                return
            if event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    return
                if event.key == K_LEFT:
                    if task == 0: # tile
                        current_tile -= 1
                        if current_tile < 0:
                            current_tile = len(tiles)-1
                    elif task == 1: # number
                        number -= 1
                        if number < 0:
                            number = 9
                    elif task == 2: # op
                        op -= 1
                        if op < 0:
                            op = 4
                            
                if event.key == K_RIGHT:
                    if task == 0:
                        current_tile += 1
                        if current_tile >= len(tiles):
                            current_tile = 0
                    if task == 1:
                        number += 1
                        if number > 9:
                            number = 0
                    if task == 2:
                        op += 1
                        if op > 4:
                            op = 0

                if event.key == K_UP:                            
                    task -= 1
                    if task < 0:
                        task = 3
                if event.key == K_DOWN:
                    task += 1
                    if task > 3:
                        task = 0
                if event.key == K_F2:
                    mapa.save(filename)
                if event.key == K_F3:
                    mapa, filename = load_map()
                if event.key == K_F4:
                    mapa = edit_map()
            
                
        if Input.down("MB1"):
            p = oy*mapa.width + ox
            if task == 0:
                if mapa.mapData[p] is None:
                    mapa.mapData[p] = map.oTile(0,0,False)
                mapa.mapData[p].imageID = tiles[current_tile].imageID
                mapa.mapData[p].height  = tiles[current_tile].height
                mapa.mapData[p].walkable = tiles[current_tile].walkable
            elif task == 1:
                mapa.mapData[p].number = number
            elif task == 2:
                mapa.mapData[p].op = op
            elif task == 3:
                mapa.mapData[p].portal = True
        if Input.down("MB3"):
            p = oy*mapa.width + ox
            if task == 0:
                mapa.mapData[p] = None
            elif task == 1:
                mapa.mapData[p].number = -1
            elif task == 2:
                mapa.mapData[p].op = -1
            elif task == 3:
                mapa.mapData[p].portal = False
        
        core.screen.fill((180,180,180,255))
        
        mapa.draw(core.screen)
        if task == 0:
            core.screen.blit( mapa.tileset[tiles[current_tile].imageID], (ox*tw,oy*th), special_flags=BLEND_RGBA_MULT )
        elif task == 1:
            core.screen.blit(mapa.tileset[10+number], (ox*tw, oy*th-25))
        elif task == 2:
            core.screen.blit(mapa.tileset[20+op], (ox*tw, oy*th-25))
        elif task == 3:
            core.screen.blit(mapa.tileset[8], (ox*tw, oy*th-25))
        core.screen.blit( selector, (ox*tw,oy*th-25) )
            
        
        core.fonts.Draw(core.screen, 'arial', 24, 'x, y: ' + str(ox) + ', ' + str(oy), (10, 10), (255,255,255,255))
        
        core.updateScreen()
        